@@ -132,10 +132,15 @@ void KMouthApp::initActions() {
    phrasebookEdit = new KAction(i18n("&Edit..."), 0, 0, this, SLOT(slotEditPhrasebook()), actionCollection(),"phrasebook_edit");
 
 // The "Options" menu
+   viewMenuBar = KStdAction::showMenubar(this, SLOT(slotViewMenuBar()), actionCollection());
    viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection());
    viewToolBar->setStatusText(i18n("Enables/disables the toolbar"));
    viewToolBar->setWhatsThis (i18n("Enables/disables the toolbar"));
 
+   viewPhrasebookBar = new KToggleAction (i18n("Show P&hrasebook Bar"), 0, 0, this, SLOT(slotViewPhrasebookBar()), actionCollection(), "showPhrasebookBar");
+   viewPhrasebookBar->setStatusText(i18n("Enables/disables the phrasebook bar"));
+   viewPhrasebookBar->setWhatsThis (i18n("Enables/disables the phrasebook bar"));
+
    viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection());
    viewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));
    viewStatusBar->setWhatsThis (i18n("Enables/disables the statusbar"));
@@ -152,9 +157,9 @@ void KMouthApp::initActions() {
    phraseListSpeak->setStatusText(i18n("Speaks the currently selected phrases in the history"));
    phraseListSpeak->setWhatsThis (i18n("Speaks the currently selected phrases in the history"));
 
-   phraseListRemove = new KAction (i18n("&Remove"), "remove", 0, phraseList, SLOT(removeListSelection()), actionCollection(), "phraselist_remove");
-   phraseListRemove->setStatusText(i18n("Removes the currently selected phrases from the history"));
-   phraseListRemove->setWhatsThis (i18n("Removes the currently selected phrases from the history"));
+   phraseListRemove = new KAction (i18n("&Delete"), "editdelete", 0, phraseList, SLOT(removeListSelection()), actionCollection(), "phraselist_remove");
+   phraseListRemove->setStatusText(i18n("Deletes the currently selected phrases from the history"));
+   phraseListRemove->setWhatsThis (i18n("Deletes the currently selected phrases from the history"));
 
    phraseListCut = new KAction (i18n("Cu&t"),   "editcut", 0, phraseList, SLOT(cutListSelection()),    actionCollection(), "phraselist_cut");
    phraseListCut->setStatusText(i18n("Cuts the currently selected phrases from the history and puts them to the clipboard"));
@@ -168,7 +173,7 @@ void KMouthApp::initActions() {
    phraselistSelectAll->setStatusText(i18n("Selects all phrases in the history"));
    phraselistSelectAll->setWhatsThis (i18n("Selects all phrases in the history"));
 
-   phraselistDeselectAll = new KAction (i18n("&Deselect All Entries"), 0, 0, phraseList, SLOT(deselectAllEntries()), actionCollection(),"phraselist_deselect_all");
+   phraselistDeselectAll = new KAction (i18n("D&eselect All Entries"), 0, 0, phraseList, SLOT(deselectAllEntries()), actionCollection(),"phraselist_deselect_all");
    phraselistDeselectAll->setStatusText(i18n("Deselects all phrases in the history"));
    phraselistDeselectAll->setWhatsThis (i18n("Deselects all phrases in the history"));
 
@@ -188,7 +193,7 @@ void KMouthApp::initStatusBar()
 }
 
 void KMouthApp::initPhraseList()
-{ 
+{
   ////////////////////////////////////////////////////////////////////
   // create the main widget here that is managed by KTMainWindow's view-region and
   // connect the widget to your document to display document contents.
@@ -209,7 +214,9 @@ void KMouthApp::saveOptions() {
    if (isConfigured) {
       config->setGroup("General Options");
       config->writeEntry("Geometry", size());
+      config->writeEntry("Show Menubar", viewMenuBar->isChecked());
       config->writeEntry("Show Toolbar", viewToolBar->isChecked());
+      config->writeEntry("Show Phrasebook Bar", viewPhrasebookBar->isChecked());
       config->writeEntry("Show Statusbar",viewStatusBar->isChecked());
       config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());
 
@@ -227,10 +234,17 @@ void KMouthApp::readOptions()
   config->setGroup("General Options");
 
   // bar status settings
+  bool bViewMenubar = config->readBoolEntry("Show Menubar", true);
+  viewMenuBar->setChecked(bViewMenubar);
+  slotViewMenuBar();
+
   bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
   viewToolBar->setChecked(bViewToolbar);
   slotViewToolBar();
 
+  bool bViewPhrasebookbar = config->readBoolEntry("Show Phrasebook Bar", true);
+  viewPhrasebookBar->setChecked(bViewPhrasebookbar);
+
   bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
   viewStatusBar->setChecked(bViewStatusbar);
   slotViewStatusBar();
@@ -253,9 +267,9 @@ void KMouthApp::readOptions()
   toolBar("phrasebookBar")->applySettings (config, "phrasebookBar");
 
   QString standardBook = KApplication::kApplication()->dirs()->findResource("appdata", "standard.phrasebook");
-  if (!standardBook.isNull() && !standardBook.isEmpty()) {
+  if (!standardBook.isEmpty()) {
      PhraseBook book;
-     book.open(standardBook);
+     book.open(KURL::fromPathOrURL( standardBook ));
      slotPhrasebookConfirmed(book);
   }
   if (phraseList != 0)
@@ -277,9 +291,9 @@ void KMouthApp::enableMenuEntries(bool existSelectedEntries, bool existDeselecte
   bool existEntries = existSelectedEntries | existDeselectedEntries;
   fileSaveAs->setEnabled (existEntries);
   filePrint->setEnabled (existEntries);
-  
+
   phraselistSelectAll->setEnabled (existDeselectedEntries);
-  
+
   phraselistDeselectAll->setEnabled (existSelectedEntries);
   phraseListSpeak->setEnabled (existSelectedEntries);
   phraseListRemove->setEnabled (existSelectedEntries);
@@ -337,7 +351,7 @@ void KMouthApp::slotFileQuit()
          break;
       memberList->removeRef(w);
     }
-  }	
+  }
 }
 
 void KMouthApp::slotEditPhrasebook () {
@@ -355,6 +369,17 @@ void KMouthApp::slotEditPhrasebook () {
    phraseBookDialog->raise();
 }
 
+void KMouthApp::slotViewMenuBar() {
+   slotStatusMsg(i18n("Toggling menubar..."));
+
+   if(!viewMenuBar->isChecked())
+      menuBar()->hide();
+   else
+      menuBar()->show();
+
+  slotStatusMsg(i18n("Ready."));
+}
+
 void KMouthApp::slotViewToolBar()
 {
   slotStatusMsg(i18n("Toggling toolbar..."));
@@ -367,7 +392,24 @@ void KMouthApp::slotViewToolBar()
   else
   {
     toolBar("mainToolBar")->show();
-  }		
+  }
+
+  slotStatusMsg(i18n("Ready."));
+}
+
+void KMouthApp::slotViewPhrasebookBar()
+{
+  slotStatusMsg(i18n("Toggling phrasebook bar..."));
+  ///////////////////////////////////////////////////////////////////
+  // turn Toolbar on or off
+  if(!viewPhrasebookBar->isChecked())
+  {
+    toolBar("phrasebookBar")->hide();
+  }
+  else
+  {
+    toolBar("phrasebookBar")->show();
+  }
 
   slotStatusMsg(i18n("Ready."));
 }
@@ -420,7 +462,7 @@ void KMouthApp::slotPhrasebookConfirmed (PhraseBook &book) {
 
    QString bookLocation = KApplication::kApplication()->dirs()->saveLocation ("appdata", "/");
    if (!bookLocation.isNull() && !bookLocation.isEmpty()) {
-      book.save (bookLocation + "standard.phrasebook");
+      book.save (KURL::fromPathOrURL( bookLocation + "standard.phrasebook" ));
    }
 }
 
